@@ -1,6 +1,6 @@
 /* Dynamic architecture support for GDB, the GNU debugger.
 
-   Copyright 1998, 1999, 2000, 2002, 2003, 2004 Free Software
+   Copyright (C) 1998, 1999, 2000, 2002, 2003, 2004 Free Software
    Foundation, Inc.
 
    This file is part of GDB.
@@ -17,8 +17,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
 #ifndef GDBARCH_UTILS_H
 #define GDBARCH_UTILS_H
@@ -38,8 +38,8 @@ extern int gdbarch_debug;
 enum return_value_convention legacy_return_value (struct gdbarch *gdbarch,
 						  struct type *valtype,
 						  struct regcache *regcache,
-						  void *readbuf,
-						  const void *writebuf);
+						  gdb_byte *readbuf,
+						  const gdb_byte *writebuf);
 
 /* Implementation of extract return value that grubs around in the
    register cache.  */
@@ -96,8 +96,6 @@ extern CORE_ADDR generic_skip_trampoline_code (CORE_ADDR pc);
 extern CORE_ADDR generic_skip_solib_resolver (struct gdbarch *gdbarch,
 					      CORE_ADDR pc);
 
-extern int generic_in_solib_call_trampoline (CORE_ADDR pc, char *name);
-
 extern int generic_in_solib_return_trampoline (CORE_ADDR pc, char *name);
 
 extern int generic_in_function_epilogue_p (struct gdbarch *gdbarch, CORE_ADDR pc);
@@ -119,6 +117,9 @@ extern int generic_convert_register_p (int regnum, struct type *type);
 extern int default_stabs_argument_has_addr (struct gdbarch *gdbarch,
 					    struct type *type);
 
+extern int generic_instruction_nullified (struct gdbarch *gdbarch,
+					  struct regcache *regcache);
+
 /* For compatibility with older architectures, returns
    (LEGACY_SIM_REGNO_IGNORE) when the register doesn't have a valid
    name.  */
