@@ -1,5 +1,5 @@
 /* GNU/Linux/ARM specific low level interface, for the remote server for GDB.
-   Copyright 1995, 1996, 1998, 1999, 2000, 2001, 2002
+   Copyright (C) 1995, 1996, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005
    Free Software Foundation, Inc.
 
    This file is part of GDB.
@@ -16,8 +16,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
 #include "server.h"
 #include "linux-low.h"
@@ -47,11 +47,15 @@ arm_cannot_fetch_register (int regno)
   return (regno >= arm_num_regs);
 }
 
+extern int debug_threads;
+
 static CORE_ADDR
 arm_get_pc ()
 {
   unsigned long pc;
   collect_register_by_name ("pc", &pc);
+  if (debug_threads)
+    fprintf (stderr, "stop pc is %08lx\n", pc);
   return pc;
 }
 
@@ -66,17 +70,27 @@ arm_set_pc (CORE_ADDR pc)
 static const unsigned long arm_breakpoint = 0xef9f0001;
 #define arm_breakpoint_len 4
 
+/* For new EABI binaries.  We recognize it regardless of which ABI
+   is used for gdbserver, so single threaded debugging should work
+   OK, but for multi-threaded debugging we only insert the current
+   ABI's breakpoint instruction.  For now at least.  */
+static const unsigned long arm_eabi_breakpoint = 0xe7f001f0;
+
 static int
 arm_breakpoint_at (CORE_ADDR where)
 {
   unsigned long insn;
 
-  (*the_target->read_memory) (where, (char *) &insn, 4);
+  (*the_target->read_memory) (where, (unsigned char *) &insn, 4);
   if (insn == arm_breakpoint)
     return 1;
 
+  if (insn == arm_eabi_breakpoint)
+    return 1;
+
   /* If necessary, recognize more trap instructions here.  GDB only uses the
-     one.  */
+     two.  */
+
   return 0;
 }
 
@@ -98,7 +112,11 @@ struct linux_target_ops the_low_target = {
   arm_cannot_store_register,
   arm_get_pc,
   arm_set_pc,
-  (const char *) &arm_breakpoint,
+#ifndef __ARM_EABI__
+  (const unsigned char *) &arm_breakpoint,
+#else
+  (const unsigned char *) &arm_eabi_breakpoint,
+#endif
   arm_breakpoint_len,
   arm_reinsert_addr,
   0,
