@@ -1,5 +1,5 @@
 /* Handling of inferior events for the event loop for GDB, the GNU debugger.
-   Copyright 1999 Free Software Foundation, Inc.
+   Copyright (C) 1999 Free Software Foundation, Inc.
    Written by Elena Zannoni <ezannoni@cygnus.com> of Cygnus Solutions.
 
    This file is part of GDB.
@@ -16,8 +16,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA. */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA. */
 
 #include "defs.h"
 #include "inferior.h"		/* For fetch_inferior_event. */
@@ -26,6 +26,7 @@
 #include "event-top.h"
 #include "inf-loop.h"
 #include "remote.h"
+#include "exceptions.h"
 
 static int fetch_inferior_event_wrapper (gdb_client_data client_data);
 static void complete_execution (void);
@@ -47,7 +48,7 @@ inferior_event_handler (enum inferior_event_type event_type,
   switch (event_type)
     {
     case INF_ERROR:
-      printf_unfiltered ("error detected from target.\n");
+      printf_unfiltered (_("error detected from target.\n"));
       target_async (NULL, 0);
       pop_target ();
       discard_all_continuations ();
@@ -93,7 +94,7 @@ inferior_event_handler (enum inferior_event_type event_type,
 
     case INF_TIMER:
     default:
-      printf_unfiltered ("Event type not recognized.\n");
+      printf_unfiltered (_("Event type not recognized.\n"));
       break;
     }
 }
@@ -127,6 +128,6 @@ complete_execution (void)
   else
     {
       if (exec_done_display_p)
-	printf_unfiltered ("completed.\n");
+	printf_unfiltered (_("completed.\n"));
     }
 }
