@@ -1,6 +1,6 @@
 /* Support for GDB maintenance commands.
 
-   Copyright 1992, 1993, 1994, 1995, 1996, 1997, 1999, 2000, 2001,
+   Copyright (C) 1992, 1993, 1994, 1995, 1996, 1997, 1999, 2000, 2001,
    2002, 2003, 2004 Free Software Foundation, Inc.
 
    Written by Fred Fish at Cygnus Support.
@@ -19,8 +19,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
 
 #include "defs.h"
@@ -44,8 +44,6 @@ extern void _initialize_maint_cmds (void);
 
 static void maintenance_command (char *, int);
 
-static void maintenance_dump_me (char *, int);
-
 static void maintenance_internal_error (char *args, int from_tty);
 
 static void maintenance_demangle (char *, int);
