@@ -1,6 +1,6 @@
 /* GDB routines for manipulating objfiles.
 
-   Copyright 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000,
+   Copyright (C) 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000,
    2001, 2002, 2003, 2004 Free Software Foundation, Inc.
 
    Contributed by Cygnus Support, using pieces from other GDB modules.
@@ -19,8 +19,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
 /* This file contains support routines for creating, manipulating, and
    destroying objfile structures. */
@@ -33,7 +33,7 @@
 #include "gdb-stabs.h"
 #include "target.h"
 #include "bcache.h"
-
+#include "mdebugread.h"
 #include "gdb_assert.h"
 #include <sys/types.h>
 #include "gdb_stat.h"
@@ -45,6 +45,7 @@
 #include "breakpoint.h"
 #include "block.h"
 #include "dictionary.h"
+#include "source.h"
 
 /* Prototypes for local functions */
 
@@ -191,7 +192,7 @@ allocate_objfile (bfd *abfd, int flags)
 
       if (build_objfile_section_table (objfile))
 	{
-	  error ("Can't find the file sections in `%s': %s",
+	  error (_("Can't find the file sections in `%s': %s"),
 		 objfile->name, bfd_errmsg (bfd_get_error ()));
 	}
     }
@@ -305,7 +306,7 @@ put_objfile_before (struct objfile *objfile, struct objfile *before_this)
     }
   
   internal_error (__FILE__, __LINE__,
-		  "put_objfile_before: before objfile not in list");
+		  _("put_objfile_before: before objfile not in list"));
 }
 
 /* Put OBJFILE at the front of the list.  */
@@ -357,7 +358,7 @@ unlink_objfile (struct objfile *objfile)
     }
 
   internal_error (__FILE__, __LINE__,
-		  "unlink_objfile: objfile already unlinked");
+		  _("unlink_objfile: objfile already unlinked"));
 }
 
 
@@ -392,6 +393,10 @@ free_objfile (struct objfile *objfile)
       objfile->separate_debug_objfile_backlink->separate_debug_objfile = NULL;
     }
   
+  /* Remove any references to this objfile in the global value
+     lists.  */
+  preserve_values (objfile);
+
   /* First do any symbol file specific actions required when we are
      finished with a particular symbol file.  Note that if the objfile
      is using reusable symbol information (via mmalloc) then each of
@@ -410,7 +415,7 @@ free_objfile (struct objfile *objfile)
     {
       char *name = bfd_get_filename (objfile->obfd);
       if (!bfd_close (objfile->obfd))
-	warning ("cannot close \"%s\": %s",
+	warning (_("cannot close \"%s\": %s"),
 		 name, bfd_errmsg (bfd_get_error ()));
       xfree (name);
     }
@@ -432,10 +437,24 @@ free_objfile (struct objfile *objfile)
      is unknown, but we play it safe for now and keep each action until
      it is shown to be no longer needed. */
 
-  /* I *think* all our callers call clear_symtab_users.  If so, no need
-     to call this here.  */
+  /* Not all our callers call clear_symtab_users (objfile_purge_solibs,
+     for example), so we need to call this here.  */
   clear_pc_function_cache ();
 
+  /* Check to see if the current_source_symtab belongs to this objfile,
+     and if so, call clear_current_source_symtab_and_line. */
+
+  {
+    struct symtab_and_line cursal = get_current_source_symtab_and_line ();
+    struct symtab *s;
+
+    ALL_OBJFILE_SYMTABS (objfile, s)
+      {
+	if (s == cursal.symtab)
+	  clear_current_source_symtab_and_line ();
+      }
+  }
+
   /* The last thing we do is free the objfile struct itself. */
 
   objfile_free_data (objfile);
@@ -555,15 +574,6 @@ objfile_relocate (struct objfile *objfile, struct section_offsets *new_offsets)
 		  SYMBOL_VALUE_ADDRESS (sym) +=
 		    ANOFFSET (delta, SYMBOL_SECTION (sym));
 		}
-#ifdef MIPS_EFI_SYMBOL_NAME
-	      /* Relocate Extra Function Info for ecoff.  */
-
-	      else if (SYMBOL_CLASS (sym) == LOC_CONST
-		       && SYMBOL_DOMAIN (sym) == LABEL_DOMAIN
-		       && strcmp (DEPRECATED_SYMBOL_NAME (sym), MIPS_EFI_SYMBOL_NAME) == 0)
-		ecoff_relocate_efi (sym, ANOFFSET (delta,
-						   s->block_line_section));
-#endif
 	    }
 	}
     }
