@@ -1,7 +1,7 @@
 /* Remote debugging for the ARM RDP interface.
 
-   Copyright 1994, 1995, 1998, 1999, 2000, 2001, 2002, 2003 Free
-   Software Foundation, Inc.
+   Copyright (C) 1994, 1995, 1998, 1999, 2000, 2001, 2002, 2003, 2006
+   Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -17,8 +17,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  
 
 
  */
@@ -172,7 +172,7 @@ static int timeout = 2;
 static char *commandline = NULL;
 
 static int
-remote_rdp_xfer_inferior_memory (CORE_ADDR memaddr, char *myaddr, int len,
+remote_rdp_xfer_inferior_memory (CORE_ADDR memaddr, gdb_byte *myaddr, int len,
 				 int write, 
 				 struct mem_attrib *attrib,
 				 struct target_ops *target);
@@ -193,7 +193,7 @@ get_byte (void)
       if (timeout == 0)
 	return (unsigned char) c;
 
-      error ("Timeout reading from remote_system");
+      error (_("Timeout reading from remote_system"));
     }
 
   return c;
@@ -387,7 +387,7 @@ rdp_init (int cold, int tty)
 
   if (!sync)
     {
-      error ("Couldn't reset the board, try pressing the reset button");
+      error (_("Couldn't reset the board, try pressing the reset button"));
     }
 }
 
@@ -454,10 +454,10 @@ send_rdp (char *template,...)
 	      printf_unfiltered ("RDP: Unimplemented message\n");
 	      return;
 	    case 255:
-	      error ("Command garbled");
+	      error (_("Command garbled"));
 	      break;
 	    default:
-	      error ("Corrupt reply from target");
+	      error (_("Corrupt reply from target"));
 	      break;
 	    }
 	  break;
@@ -495,13 +495,13 @@ send_rdp (char *template,...)
 	  *pi = get_byte ();
 	  break;
 	default:
-	  internal_error (__FILE__, __LINE__, "failed internal consistency check");
+	  internal_error (__FILE__, __LINE__, _("failed internal consistency check"));
 	}
     }
   va_end (alist);
 
   if (dst != buf)
-    internal_error (__FILE__, __LINE__, "failed internal consistency check");
+    internal_error (__FILE__, __LINE__, _("failed internal consistency check"));
 }
 
 
@@ -645,7 +645,7 @@ remote_rdp_fetch_register (int regno)
 	{
 	  printf ("Help me with fetch reg %d\n", regno);
 	}
-      supply_register (regno, buf);
+      regcache_raw_supply (current_regcache, regno, buf);
     }
 }
 
@@ -791,9 +791,6 @@ argsin;
 #define SWI_GenerateError               0x71
 
 
-#ifndef O_BINARY
-#define O_BINARY 0
-#endif
 
 static int translate_open_mode[] =
 {
@@ -988,7 +985,7 @@ handle_swi (void)
 	  break;
 
 	default:
-	  error ("Unimplemented SWI argument");
+	  error (_("Unimplemented SWI argument"));
 	}
 
       type = type >> 2;
@@ -1053,8 +1050,10 @@ rdp_execute (void)
 }
 
 static int
-remote_rdp_insert_breakpoint (CORE_ADDR addr, char *save)
+remote_rdp_insert_breakpoint (struct bp_target_info *bp_tgt)
 {
+  CORE_ADDR addr = bp_tgt->placed_address;
+
   int res;
   if (ds.rdi_level > 0)
     {
@@ -1062,7 +1061,7 @@ remote_rdp_insert_breakpoint (CORE_ADDR addr, char *save)
 		RDP_SET_BREAK,
 		addr,
 		RDP_SET_BREAK_TYPE_PC_EQUAL | RDP_SET_BREAK_TYPE_GET_HANDLE,
-		save,
+		bp_tgt->shadow_contents,
 		&res);
     }
   else
@@ -1077,14 +1076,15 @@ remote_rdp_insert_breakpoint (CORE_ADDR addr, char *save)
 }
 
 static int
-remote_rdp_remove_breakpoint (CORE_ADDR addr, char *save)
+remote_rdp_remove_breakpoint (struct bp_target_info *bp_tgt)
 {
+  CORE_ADDR addr = bp_tgt->placed_address;
   int res;
   if (ds.rdi_level > 0)
     {
       send_rdp ("b-p-S-B",
 		RDP_CLEAR_BREAK,
-		save, 4,
+		bp_tgt->shadow_contents, 4,
 		&res);
     }
   else
@@ -1111,12 +1111,12 @@ rdp_step (void)
     }
   else
     {
-      char handle[4];
+      void *b;
       CORE_ADDR pc = read_register (ARM_PC_REGNUM);
       pc = arm_get_next_pc (pc);
-      remote_rdp_insert_breakpoint (pc, handle);
+      b = deprecated_insert_raw_breakpoint (pc);
       rdp_execute ();
-      remote_rdp_remove_breakpoint (pc, handle);
+      deprecated_remove_raw_breakpoint (b);
     }
 }
 
@@ -1126,7 +1126,7 @@ remote_rdp_open (char *args, int from_tty)
   int not_icebreaker;
 
   if (!args)
-    error_no_arg ("serial port device name");
+    error_no_arg (_("serial port device name"));
 
   baud_rate = 9600;
 
@@ -1181,7 +1181,7 @@ remote_rdp_open (char *args, int from_tty)
   flush_cached_frames ();
   registers_changed ();
   stop_pc = read_pc ();
-  print_stack_frame (get_selected_frame (), 0, SRC_AND_LOC);
+  print_stack_frame (get_selected_frame (NULL), 0, SRC_AND_LOC);
 }
 
 
@@ -1264,7 +1264,7 @@ remote_rdp_prepare_to_store (void)
    Returns the number of bytes transferred. */
 
 static int
-remote_rdp_xfer_inferior_memory (CORE_ADDR memaddr, char *myaddr, int len,
+remote_rdp_xfer_inferior_memory (CORE_ADDR memaddr, gdb_byte *myaddr, int len,
 				 int write, struct mem_attrib *attrib,
 				 struct target_ops *target)
 {
@@ -1357,7 +1357,7 @@ remote_rdp_create_inferior (char *exec_file, char *allargs, char **env,
   CORE_ADDR entry_point;
 
   if (exec_file == 0 || exec_bfd == 0)
-    error ("No executable file specified.");
+    error (_("No executable file specified."));
 
   entry_point = (CORE_ADDR) bfd_get_start_address (exec_bfd);
 
@@ -1376,8 +1376,7 @@ remote_rdp_create_inferior (char *exec_file, char *allargs, char **env,
      ** so we don't bother to look for MEMSIZE in the environment.
    */
 
-  /* Let's go! */
-  proceed (entry_point, TARGET_SIGNAL_DEFAULT, 0);
+  write_pc (entry_point);
 }
 
 /* Attach doesn't need to do anything */
@@ -1405,7 +1404,7 @@ init_remote_rdp_ops (void)
   remote_rdp_ops.to_fetch_registers = remote_rdp_fetch_register;
   remote_rdp_ops.to_store_registers = remote_rdp_store_register;
   remote_rdp_ops.to_prepare_to_store = remote_rdp_prepare_to_store;
-  remote_rdp_ops.to_xfer_memory = remote_rdp_xfer_inferior_memory;
+  remote_rdp_ops.deprecated_xfer_memory = remote_rdp_xfer_inferior_memory;
   remote_rdp_ops.to_files_info = remote_rdp_files_info;
   remote_rdp_ops.to_insert_breakpoint = remote_rdp_insert_breakpoint;
   remote_rdp_ops.to_remove_breakpoint = remote_rdp_remove_breakpoint;
