@@ -1,5 +1,5 @@
-/* Remote serial interface using Hitachi E7000 PC ISA card in a PC
-   Copyright 1994, 1996, 1997, 1998, 1999, 2000
+/* Remote serial interface using Renesas E7000 PC ISA card in a PC
+   Copyright (C) 1994, 1996, 1997, 1998, 1999, 2000
    Free Software Foundation, Inc.
 
    This file is part of GDB.
@@ -16,8 +16,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
 #include "defs.h"
 #if defined __GO32__ || defined _WIN32
@@ -164,16 +164,16 @@ e7000pc_init (void)
 			       sigs[try].addr);
 	      return 1;
 	    }
-	  error ("The E7000 PC board is working, but the E7000 is turned off.\n");
+	  error (_("The E7000 PC board is working, but the E7000 is turned off."));
 	  return 0;
 	}
     }
 
-  error ("GDB cannot connect to the E7000 PC board, check that it is installed\n\
+  error (_("GDB cannot connect to the E7000 PC board, check that it is installed\n\
 and that the switch settings are correct.  Some other DOS programs can \n\
 stop the board from working.  Try starting from a very minimal boot, \n\
 perhaps you need to disable EMM386 over the region where the board has\n\
-its I/O space, remove other unneeded cards, etc etc\n");
+its I/O space, remove other unneeded cards, etc etc\n"));
   return 0;
 
 }
