@@ -1,6 +1,6 @@
 /* Include file for stabs debugging format support functions.
-   Copyright 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995,
-   1996, 1997, 1999, 2000 Free Software Foundation, Inc.
+   Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995,
+   1996, 1997, 1999, 2000, 2001, 2002, 2003 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -16,8 +16,10 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
+
+struct objfile;
 
 /* Definitions, prototypes, etc for stabs debugging format support
    functions.
@@ -128,19 +130,10 @@ EXTERN int n_this_object_header_files;
 
 EXTERN int n_allocated_this_object_header_files;
 
-extern struct complaint unknown_symtype_complaint;
-extern struct complaint unknown_symchar_complaint;
-
-extern struct type *read_type (char **, struct objfile *);
-
 extern void cleanup_undefined_types (void);
 
-extern struct type **dbx_lookup_type (int[2]);
-
 extern long read_number (char **, int);
 
-extern void add_undefined_type (struct type *);
-
 extern struct symbol *define_symbol (CORE_ADDR, char *, int, int,
 				     struct objfile *);
 
@@ -154,9 +147,6 @@ extern void end_stabs (void);
 
 extern void finish_global_stabs (struct objfile *objfile);
 
-
-EXTERN int os9k_stabs;
-
 /* COFF files can have multiple .stab sections, if they are linked
    using --split-by-reloc.  This linked list is used to pass the
    information into the functions in dbxread.c.  */
@@ -182,15 +172,14 @@ extern struct partial_symtab *end_psymtab (struct partial_symtab *pst,
 					   int number_dependencies,
 					   int textlow_not_set);
 
-extern void
-process_one_symbol (int, int, CORE_ADDR, char *,
-		    struct section_offsets *, struct objfile *);
+extern void process_one_symbol (int, int, CORE_ADDR, char *,
+				struct section_offsets *, struct objfile *);
 
-extern void elfstab_build_psymtabs
-  (struct objfile *objfile,
-   int mainline,
-   file_ptr staboff, unsigned int stabsize,
-   file_ptr stabstroffset, unsigned int stabstrsize);
+extern void elfstab_build_psymtabs (struct objfile *objfile,
+				    int mainline,
+				    asection *stabsect,
+				    file_ptr stabstroffset,
+				    unsigned int stabstrsize);
 
 extern void coffstab_build_psymtabs
   (struct objfile *objfile,
@@ -205,20 +194,12 @@ extern void stabsect_build_psymtabs
 
 extern void elfstab_offset_sections (struct objfile *,
 				     struct partial_symtab *);
-
-extern void process_later
-  (struct symbol *, char *,
-   int (*f) (struct objfile *, struct symbol *, char *));
-
 extern int symbol_reference_defined (char **);
 
 extern void ref_add (int, struct symbol *, char *, CORE_ADDR);
 
 extern struct symbol *ref_search (int);
 
-extern int resolve_cfront_continuation
-  (struct objfile *objfile, struct symbol *sym, char *p);
-
 extern void free_header_files (void);
 
 extern void init_header_files (void);
