@@ -1,5 +1,5 @@
 /* Data structures for RS/6000 shared libraries, for GDB.
-   Copyright 1991, 1992, 1993, 1994, 1996, 1997, 2000
+   Copyright (C) 1991, 1992, 1993, 1994, 1996, 1997, 2000
    Free Software Foundation, Inc.
 
    This file is part of GDB.
@@ -16,8 +16,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
 /* The vmap struct is used to describe the virtual address space of
    the target we are manipulating.  The first entry is always the "exec"
@@ -27,6 +27,9 @@
    shared libraries are loaded, or a "load" system call, followed by the
    user's issuance of a "load" command.  */
 
+#ifndef XCOFFSOLIB_H
+#define XCOFFSOLIB_H
+
 struct vmap
   {
     struct vmap *nxt;		/* ptr to next in chain                 */
@@ -59,3 +62,5 @@ extern struct vmap *vmap;
 
 /* Add symbols for a vmap.  */
 extern int vmap_add_symbols (struct vmap *vp);
+
+#endif
